# encoding: utf-8

# *****************************************************************************
# Copyright 2023 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# *****************************************************************************
# *****************************************************************************
#
# (c) Copyright 2022 Micro Focus or one of its affiliates
#
# The only warranties for products and services of Micro Focus and its
# affiliates and licensors(Micro Focus) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional warranty.
# Micro Focus shall not be liable for technical or editorial errors or
# omissions contained herein.
# The information contained herein is subject to change without notice.
#
# ******************************************************************************
# File:        modules/Umc.rb
# Module:      Installation
# Summary:     Perform UMC Configuration
# Authors:     Gaurav Kumar <gaurav.kumar4@microfocus.com>
#              Srinidhi B <Srinidhi.BS@microfocus.com>
#

require "yast"

module Yast
  class UmcClass < Module
    def main
      require 'json'
      textdomain "umc"

      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Summary"
      Yast.import "Popup"
      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "Service"
      Yast.import "Edirectory"
      Yast.import "NovellUtils"
      Yast.import "NovellLDAP"
      Yast.import "String"
      Yast.import "Wizard"
      @auto_clone = false

      # Settings Map
      @Settings = {}

      @server_contest = ""
      @admin_context = ""
      @xadMode = false
      @useXad = false

      # Data was modified?
      @modified = false

      @proposal_valid = false

      @globals_configured = false

      @channelUpgradeMode = false
      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")

      @scr_conf_path = path(".sysconfig.umc")
      @umc_config_script = "/opt/novell/umc/scripts/umc-config.py"
      @umc_schema = "/opt/novell/sch/umc.sch"
      # Store the existing db hostname detected by get_config
      @existing_db_hostname = ""
      @syncError = _("Synchronization is taking more time.\nIf the UMC configuration fails, retry.")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    def ProgressNextStage(title)
      Progress.NextStage
      Progress.Title(title)

      nil
    end

    def GetAdminContext
      NovellLDAP.Read

      if NovellPatterns.IsPatternInstalled("novell-domain-services")
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
          Convert.to_string(SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))) == "yes"
          @xadMode = true
          xad_tree_admin_context = Ops.get_string(NovellLDAP.Settings,"XAD_TREE_ADMIN_CONTEXT","")
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            @useXad = true
          end
        end
      end

      if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
        Convert.to_string(SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))) == "yes"
        if Ops.get_string(Edirectory.EdirSettings, "CONFIG_EDIR_TREE_TYPE", "") == "existing" ||
          Convert.to_string(SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))) == "existing"
          @admin_context = Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_CONTEXT", "")
        else
          #For NNM FRD
          @admin_context = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        end
      else
        @admin_context = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      end
      @server_context = Ops.get_string(Edirectory.EdirSettings, "CONFIG_EDIR_SERVER_CONTEXT", "")
      if @server_context == nil || @server_context == ""
        @server_context = Convert.to_string(SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT")))
      end
      @admin_context = NovellUtils.Nds2Ldap(@admin_context)
      @server_context = NovellUtils.Nds2Ldap(@server_context)

    end

    def ReadOption(optname, replace)
      optvalue = Convert.to_string(
        SCR.Read(Ops.add(@scr_conf_path, Builtins.topath(optname)))
      )

      optvalue = "" if optvalue == nil
      optvalue = replace if optvalue == "" && replace != nil

      optvalue
    end

    # Read all umc settings
    # @return true on success
    def Read
      steps = [
        # Progress stage 1
        _("Read LDAP configuration settings"),
        # Progress stage 2
        _("Read UMC configuration settings"),
      ]

      caption = _("Initializing Unified Management Console")

      Progress.New(caption, " ", Builtins.size(steps), steps, [], "")

      return false if Abort()
      #Step 1: Read LDAP Settings
      ProgressNextStage(_("Reading LDAP configuration settings..."))
      current_progress = Progress.set(false)
      GetAdminContext()
      Progress.set(current_progress)

      #Step 2: Read UMC Configuration
      ProgressNextStage(_("Reading UMC configuration settings..."))
      current_progress = Progress.set(false)

      database_host = NovellUtils.GetDNSname
      database_port = ReadOption("CONFIG_UMC_DB_PORT", "")
      database_port = "5432" if database_port == ""

      edir_configured = Convert.to_string(SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED")))
      if edir_configured == "yes"
        tree_type = Convert.to_string(SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE")))
      elsif edir_configured == "no"
        tree_type = Ops.get_string(Edirectory.EdirSettings, "CONFIG_EDIR_TREE_TYPE", "")
      end
      Builtins.y2milestone("tree_type value is %1", tree_type)

      cmd = Builtins.sformat(
        "python3 %1 get_config",
          @umc_config_script
      )

      if tree_type == "existing"
        replica_server = Ops.get_string(Edirectory.EdirSettings, "CONFIG_EDIR_REPLICA_SERVER", "")
        if replica_server == "" || replica_server == nil
          replica_server = SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_REPLICA_SERVER"))
        end
        cmd = Ops.add(cmd, " --ldap-server #{replica_server}")
      end

      Builtins.y2milestone(
        "Executing %1",
          cmd
      )
      rmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          cmd
        )
      )
      Builtins.y2milestone(
        "Retmap for %1 is %2",
        cmd,
        rmap
      )

      stdout = NovellUtils.CleanUpString(Ops.get_string(rmap, "stdout", ""))

      Builtins.y2milestone(
        "stdout is %1",
        stdout
      )

      if Ops.get_integer(rmap, "exit", -1) == 0 &&
        Ops.greater_than(Builtins.size(stdout), 0) &&
        stdout != ""

        parsed = JSON.parse(stdout)

        if !parsed["db_instances"].nil?
          parsed["db_instances"].each do |db_instances|
            database_host = db_instances["host"]
            database_port = db_instances["port"]
          end
          @existing_db_hostname = database_host
        end

        Builtins.y2milestone(
          "Values from %1 : for database host is %2 and port is %3",
          cmd,
          database_host,
          database_port
        )
      end

      Ops.set(
        @Settings,
        "CONFIG_UMC_DB_HOSTNAME",
        ReadOption("CONFIG_UMC_DB_HOSTNAME", database_host)
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_DB_PORT",
        ReadOption("CONFIG_UMC_DB_PORT", database_port)
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_DB_USERNAME",
        ReadOption("CONFIG_UMC_DB_USERNAME", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_OPEN_DB_PORT",
        ReadOption("CONFIG_UMC_OPEN_DB_PORT", "yes")
      )
      Ops.set(
        @Settings,
        "SERVICE_CONFIGURED",
        ReadOption("SERVICE_CONFIGURED", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_EDIRAPI_PORT",
        ReadOption("CONFIG_UMC_EDIRAPI_PORT", "9010")
      )

      if Mode.update || Mode.autoupgrade || @channelUpgradeMode
        Builtins.y2milestone("UMC : In Upgrade mode and value of channelUpgradeMode is %1", @channelUpgradeMode)
        Ops.set(
          @Settings,
          "CONFIG_UMC_DB_PASSWORD",
          GetDBPwd()
        )
      end

      Builtins.y2debug("Umc:READ Complete:%1", @Settings)
      @globals_configured = true

      Progress.set(current_progress)
      return false if Abort()
      ProgressNextStage(_("Finished"))
      Progress.Finish
      true
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("UMC Database Hostname: %1"),
            Ops.get_string(@Settings, "CONFIG_UMC_DB_HOSTNAME", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("UMC Database Port: %1"),
            Ops.get_string(@Settings, "CONFIG_UMC_DB_PORT", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("UMC Database UserName: %1"),
            Ops.get_string(@Settings, "CONFIG_UMC_DB_USERNAME", "")
          )
        )
        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Write all umc settings
    # @return true on success
    def Write
      write_error = false

      Wizard.DisableAbortButton
      Builtins.y2milestone("UMC::Write started")
      pattern = "microfocus-umc"
      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end
        return true
      end

      if !Ops.get(NovellAddonInstall.products_to_configure, "umc", true)
        Builtins.y2milestone("Unified Management Console -- configure later")
        return true
      end

      if Mode.autoinst
        GetAdminContext()
      end

      steps = [
        # Progress stage 1
        _("Execute UMC configuration settings"),
        # Progress stage 2
        _("Write UMC configuration settings"),
      ]

      caption = _("Saving Unified Management Console Configuration")

      Progress.New(caption, " ", Builtins.size(steps), steps, [], "")

      return false if Abort()
      #Step 1: Execute UMC Settings
      ProgressNextStage(_("Executing UMC configuration settings..."))
      current_progress = Progress.set(false)

      if @xadMode && @useXad
        adminPasswd = Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_PASSWORD", "")
      else
        adminPasswd = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
      end

      if Ops.get_string(@Settings, "SERVICE_CONFIGURED", "no") == "no"

        if !NovellUtils.ExtendSchema(@umc_schema)
          write_error = true
        end

        Popup.ShowFeedback(
          _("Schema synchronization in progress..."),
          _("This may take a while.")
        )
        if !write_error
          count = 0
          while ( count < 5 )
            umcConfigSearch = Builtins.sformat(
              "/usr/bin/ldapsearch -x -b cn=schema -s base | grep -i umcConfig"
            )
            Builtins.y2milestone(
              "Executing %1 and count is %2",
              umcConfigSearch,
              count
            )
            retmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                umcConfigSearch
              )
            )
            Builtins.y2milestone(
              "Retmap for search is %1",
              retmap
            )
            if Ops.get_integer(retmap, "exit", -1) != 0 && count == 4
              Popup.ClearFeedback
              Popup.TimedMessage(@syncError, 10) if Mode.autoinst
              Popup.Message(@syncError) if Mode.installation || Stage.normal
              break
            elsif Ops.get_integer(retmap, "exit", -1) != 0 && count < 5
              count = count + 1
              Builtins.sleep(20000)
            elsif Ops.get_integer(retmap, "exit", -1) == 0
              Popup.ClearFeedback
              break
            end
          end
          cmd = format(
            'python3 %s update_umc_config --edir-admin-username %s --edir-admin-password \'%s\' --db-host %s --db-port %s --db-username %s --db-password \'%s\' --edir-api-port %s',
            @umc_config_script,
            @admin_context,
            adminPasswd,
            Ops.get_string(@Settings, "CONFIG_UMC_DB_HOSTNAME", ""),
            Ops.get_string(@Settings, "CONFIG_UMC_DB_PORT", ""),
            Ops.get_string(@Settings, "CONFIG_UMC_DB_USERNAME", ""),
            Ops.get_string(@Settings, "CONFIG_UMC_DB_PASSWORD", ""),
            Ops.get_string(@Settings, "CONFIG_UMC_EDIRAPI_PORT", "9010")
          )

          logCmd = format(
            'python3 %s update_umc_config --edir-admin-username %s --edir-admin-password *** --db-host %s --db-port %s --db-username %s --db-password *** --edir-api-port %s',
            @umc_config_script,
            @admin_context,
            Ops.get_string(@Settings, "CONFIG_UMC_DB_HOSTNAME", ""),
            Ops.get_string(@Settings, "CONFIG_UMC_DB_PORT", ""),
            Ops.get_string(@Settings, "CONFIG_UMC_DB_USERNAME", ""),
            Ops.get_string(@Settings, "CONFIG_UMC_EDIRAPI_PORT", "9010")
          )

          Builtins.y2milestone(
            "Executing %1",
            logCmd
          )

          rmap = Convert.to_map(
            SCR.Execute(
              path(".target.bash_output"),
              cmd
            )
          )
          Builtins.y2milestone(
            "Retmap for %1 update_umc_config is %2",
            @umc_config_script,
            rmap
          )

          if Ops.get_integer(rmap, "exit", -1) == 0
            cmd1 = Builtins.sformat(
              "python3 %1 update_oes_rest_config",
              @umc_config_script
            )
            Builtins.y2milestone(
              "Executing %1",
              cmd1
            )
            rmap1 = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                cmd1
              )
            )
            Builtins.y2milestone(
              "Retmap for %1 is %2",
              cmd1,
              rmap1
            )
            write_error = true if Ops.get_integer(rmap1, "exit", -1) != 0
          else
            write_error = true
          end
        end
      end

      Progress.set(current_progress)

      #Step 2: Write UMC Settings
      ProgressNextStage(_("Writing UMC configuration settings..."))
      current_progress = Progress.set(false)
      if !write_error
        SCR.Write(
          path(".sysconfig.umc.CONFIG_UMC_DB_HOSTNAME"),
          Ops.get_string(@Settings, "CONFIG_UMC_DB_HOSTNAME", "")
        )
        SCR.Write(
          path(".sysconfig.umc.CONFIG_UMC_DB_PORT"),
          Ops.get_string(@Settings, "CONFIG_UMC_DB_PORT", "")
        )
        SCR.Write(
          path(".sysconfig.umc.CONFIG_UMC_DB_USERNAME"),
          Ops.get_string(@Settings, "CONFIG_UMC_DB_USERNAME", "")
        )
        SCR.Write(
          path(".sysconfig.umc.CONFIG_UMC_OPEN_DB_PORT"),
          Ops.get_string(@Settings, "CONFIG_UMC_OPEN_DB_PORT", "")
        )
        SCR.Write(
          path(".sysconfig.umc.CONFIG_UMC_EDIRAPI_PORT"),
          Ops.get_string(@Settings, "CONFIG_UMC_EDIRAPI_PORT", "")
        )
        SCR.Write(path(".sysconfig.umc.SERVICE_CONFIGURED"), "yes")
        # Flush file
        SCR.Write(path(".sysconfig.umc"), nil)

        if Ops.get_string(@Settings, "CONFIG_UMC_OPEN_DB_PORT", "") == "yes"
          dbport = Ops.get_string(@Settings, "CONFIG_UMC_DB_PORT", "")
          NovellUtils.SetFirewallPort(dbport, :tcp, true)
        end
        execute_cmd = Builtins.sformat(
          "sed 's/welcome/umc/' /srv/www/htdocs/index.html -i"
        )
        Builtins.y2milestone(
          "Executing %1",
          execute_cmd
        )
        rtrnVal = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            execute_cmd
          )
        )
        Builtins.y2milestone(
          "Retmap for %1 is %2",
          execute_cmd,
          rtrnVal
        )
        Service.Restart("apache2")
      else
        Report.Warning(
          Builtins.sformat(
            _("Unable to update the umc configuration")
          )
        )
      end

      # Enable the service for edir api
      sr_list = [ "docker", "docker-edirapi" ]
      sr_list.each do |service|
        if Service.Enabled(service)
          Service.Restart(service)
        else
          Service.Enable(service)
          Service.Start(service)
        end
      end

      # Restart umc services in case of upgrade
      if Mode.update || Mode.autoupgrade || @channelUpgradeMode

        command_list = ["/opt/novell/umc/scripts/migrate_table.py", "/opt/novell/umc/scripts/umc-config.py update_conf_file"]
        command_list.each do |command|
          cmd = Builtins.sformat(
            "python3 %1",
            command
          )
          Builtins.y2milestone(
            "Executing %1",
            cmd
          )
          rmap = Convert.to_map(
            SCR.Execute(
              path(".target.bash_output"),
              cmd
            )
          )
          Builtins.y2milestone(
            "Retmap for %1 is %2",
            cmd,
            rmap
          )
        end

        service_restart = [ "microfocus-umc-server", "microfocus-umc-backend" ]
        service_restart.each do |service|
          Service.Restart(service)
        end

      end

      Builtins.y2debug("OES UMC Settings(Write): %1", @Settings)
      return false if Abort()
      Progress.set(current_progress)
      ProgressNextStage(_("Finished"))
      Progress.Finish

      true
    end

    # Get all umc settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "umc", false)
        return true
      end
      Ops.set(
        @Settings,
        "CONFIG_UMC_DB_HOSTNAME",
        Ops.get_string(settings, "umc_db_hostname", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_DB_PORT",
        Ops.get_string(settings, "umc_db_port", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_DB_USERNAME",
        Ops.get_string(settings, "umc_db_username", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_OPEN_DB_PORT",
        Ops.get_string(settings, "umc_open_db_port", "yes")
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_DB_PASSWORD",
        Ops.get_string(settings, "umc_db_password", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_UMC_EDIRAPI_PORT",
        Ops.get_string(settings, "umc_edirapi_port", "9010")
      )
      @globals_configured = true
      true
    end

    # Dump the umc settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("microfocus-umc") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "umc", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "umc", true)
        Ops.set(
          settings,
          "umc_db_hostname",
          Ops.get_string(@Settings, "CONFIG_UMC_DB_HOSTNAME", "")
        )
        Ops.set(
          settings,
          "umc_db_port",
          Ops.get_string(@Settings, "CONFIG_UMC_DB_PORT", "")
        )
        Ops.set(
          settings,
          "umc_db_username",
          Ops.get_string(@Settings, "CONFIG_UMC_DB_USERNAME", "")
        )
        Ops.set(
          settings,
          "umc_db_password",
          "Enter the database password"
        )
        Ops.set(
          settings,
          "umc_open_db_port",
          Ops.get_string(@Settings, "CONFIG_UMC_OPEN_DB_PORT", "yes")
        )
        Ops.set(
          settings,
          "umc_edirapi_port",
          Ops.get_string(@Settings, "CONFIG_UMC_EDIRAPI_PORT", "9010")
        )
      end
      deep_copy(settings)
    end

    def GetDBPwd
      if Ops.get_string(@Settings, "SERVICE_CONFIGURED", "no") == "yes"
        encrypted_pwd = format(
          'grep "^[[:blank:]]*DB_URL=" /opt/novell/umc/apps/umc-server/prod.env | sed s/^[[:blank:]]*DB_URL=// | cut -d: -f3 | cut -d@ -f1'
        )
        Builtins.y2milestone(
          "Executing %1",
          encrypted_pwd
        )
        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            encrypted_pwd
          )
        )
        if Ops.get_integer(rmap, "exit", -1) == 0
          stdout = NovellUtils.CleanUpString(Ops.get_string(rmap, "stdout", ""))
          if stdout != ""
            decrypted_pwd_cmd = format(
              'python3 %s decrypt_db_pwd --db-username \'%s\' --db-encrypted-pwd \'%s\'',
              @umc_config_script,
              Ops.get_string(@Settings, "CONFIG_UMC_DB_USERNAME", ""),
              stdout
            )
            rmap1 = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                decrypted_pwd_cmd
              )
            )
            if Ops.get_integer(rmap1, "exit", -1) == 0
              decrypted_pwd = NovellUtils.CleanUpString(Ops.get_string(rmap1, "stdout", ""))
              return decrypted_pwd if decrypted_pwd != ""
            end
          end
        end
      end
      return ""
    end

    def ValidateDB(dbHostname, dbUsername, dbPassword, dbPort)
      cmd = format(
        'python3 %s validate_db --db-host %s --db-port %s --db-username %s --db-password \'%s\'',
        @umc_config_script,
        dbHostname,
        dbPort,
        dbUsername,
        dbPassword
      )

      logCmd = format(
        'python3 %s validate_db --db-host %s --db-port %s --db-username %s --db-password *** ',
        @umc_config_script,
        dbHostname,
        dbPort,
        dbUsername
      )

      Builtins.y2milestone(
        "Executing %1",
        logCmd
      )

      rmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          cmd
        )
      )

      Builtins.y2milestone(
        "Retmap for %1 validate_db is %2",
        @umc_config_script,
        rmap
      )

      stdout = NovellUtils.CleanUpString(Ops.get_string(rmap, "stdout", ""))
      Builtins.y2milestone("Stdout message for running validate_db is = %1", stdout)

      if Ops.get_integer(rmap, "exit", -1) != 0
        if Builtins.contains(stdout, "password authentication failed")
          Report.Warning(
            Builtins.sformat(
              _("Invalid username or password.")
            )
          )
        elsif Builtins.contains(stdout, "failed: No route to host")
          Report.Warning(
            Builtins.sformat(
              _("Invalid hostname or port number.")
            )
          )
        elsif Builtins.contains(stdout, "could not translate host name")
          Report.Warning(
            Builtins.sformat(
              _("Entered hostname is incorrect.")
            )
          )
        else
          Report.Warning(
            Builtins.sformat(
              _("Unable to validate. Check y2log for more details.")
            )
          )
        end
        return false
      end
      return true
    end

    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :useXad, :type => "boolean"
    publish :variable => :channelUpgradeMode, :type => "boolean"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :variable => :Settings, :type => "map"
    publish :variable => :existing_db_hostname, :type => "string"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :Modified, :type => "boolean ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :ValidateDB, :type => "boolean (string, string, string, string)"
  end
  Umc = UmcClass.new
  Umc.main
end
